/*
 
 
 Layout for the i-state variables:
 
 istate[0][0] : Age
 istate[0][1] : Length
 istate[0][3] : Weight
 
 Layout for the environment variables:
 
 E[0] : Resource 1
 E[1] : Resource 2
 
 
 Layout for the interaction (and output) variables:
 
 // impact[0][0] = effect X1
 // impact[0][1] = effect X2
 // impact[0][2] = L
 // impact[0][3] = LM
 // impact[0][4] = L2
 // impact[0][5] = L2M
 // impact[0][6] = J
 // impact[0][7] = JM
 // impact[0][8] = A
 // impact[0][9] = AM
 
 // WEIGHT    = BETA*pow(LB,3);
 // LENGTH   = pow((WEIGHT/BETA),(1/3))
 
 //Last modification HTB 25 april 2018
 //Including conditions for starvation (not relevant in case of equilibrium)
 
 */

/*
 *===========================================================================
 * 		DEFINITION OF PROBLEM DIMENSIONS AND NUMERICAL SETTINGS
 *===========================================================================
 */
// Dimension settings: Required
#define POPULATION_NR           1
#define STAGES                  4
#define	I_STATE_DIM             3
#define	ENVIRON_DIM             2   
#define INTERACT_DIM			10
#define	PARAMETER_NR			22

// Numerical settings: Optional (default values adopted otherwise)
#define MIN_SURVIVAL		1.0E-9		// Survival at which individual is considered dead
#define MAX_AGE				10000		   // Give some absolute maximum for individual age

#define DYTOL				1.0E-6		// Variable tolerance
#define RHSTOL				1.0E-6		// Function tolerance
#define ALLOWNEGATIVE		0			// Negative solution values allowed?
#define COHORT_NR			10000			// Number of cohorts in state output
#define ESSTOL              1.0E-9     //ESS toll


/*
 *===========================================================================
 * 		DEFINITION OF ALIASES
 *===========================================================================
 */
// Define aliases for the istate variables
#define AGE			istate[0][0]
#define LENGTH		istate[0][1]
#define WEIGHT		istate[0][2]

// Define aliases for the environment variables
#define X1			E[0]
#define X2          E[1]

// Define aliases for the parameters
#define DELTA		parameter[ 0]	// Default: 0.1
#define XMAX1		parameter[ 1]	// Default: 
#define XMAX2		parameter[ 2]	// Default: 

#define LB			parameter[ 3]	// Default: 7  		Length at birth
#define LV			parameter[ 4]	// Default: 50 		Large juveniles 
#define LJ			parameter[ 5]	// Default: 110 	Length at maturation

#define H           parameter[ 6]   // Default: 10      Handling time (=1/Hc)
#define BETA		parameter[ 7]	// Default: 9.0E-6 	Weight-length scaling constant
#define AMAX		parameter[ 8]	// Default: 6.66 	Max attack rate

#define GAMMA		parameter[ 9]	// Default: 0.006 	Growth rate in length scaling constant
#define LM          parameter[10]   // Default: 300     Maximum length an individual can reach 
#define AC          parameter[11]   // Default: 0.003   Max. adult fecundity 

#define MU			parameter[12]	// Default: 0.01  	Background mortality rate
#define PSI         parameter[13]   // Default: 1       Specialization on X2
#define AMIN        parameter[14]   // Default: 0.66    Min attack rate
#define META        parameter[15]   // Default: 0       Extent of meta
#define QS          parameter[16]   // Default  0.2     Prop. of weight loss
#define RHO         parameter[17]   // Default  0.5     Prob. to die
#define LC          parameter[18]   // Default  0.0001  Size at meta
#define SIGMA       parameter[19]   // Default  10      Switch speed
#define mu_s        parameter[20]   // Default: 0.2        Extrasterfte
#define Tqs         parameter[21]   // Default: 0.3     Threshold extra starvation




/*
 *===========================================================================
 * 		DEFINITION OF NAMES AND DEFAULT VALUES OF THE PARAMETERS
 *===========================================================================
 */
// At least two parameters should be specified in this array
char  *parameternames[PARAMETER_NR] =
{ "Delta", "Xmax1","Xmax2","Lb", "Lv", "Lj","H", "Beta", "MaxA", "Gamma", "LM","Ac", "Mu",  "Psi",
    "Amin","Meta","QS","RHO", "LC", "SIGMA", "mu_s","Threshold_qs"};

// These are the default parameters values
double	parameter[PARAMETER_NR] =
{0.1, 1*10^-4,  1*10^-4, 7.0, 40.0, 110.0, 10, 0.009,  6.6666667, 0.006, 300, 0.003, 0.01,  0, 0.66667,0,0.2,0.5,10,10,0.2,0.3};


/*
 *===========================================================================
 * 		DEFINITION OF THE LIFE HISTORY MODELS FOLLOWS BELOW
 *===========================================================================
 * Specify the number of states at birth for the individuals in all structured
 * populations in the problem in the vector BirthStates[].
 *===========================================================================
 */

void SetBirthStates(int BirthStates[POPULATION_NR], double E[])
{
    BirthStates[0] = 1;
    
    return;
}


/*
 *===========================================================================
 * Specify all the possible states at birth for all individuals in all
 * structured populations in the problem. BirthStateNr represents the index of
 * the state of birth to be specified. Each state at birth should be a single,
 * constant value for each i-state variable.
 *
 * Notice that the first index of the variable 'istate[][]' refers to the
 * number of the structured population, the second index refers to the
 * number of the individual state variable. The interpretation of the latter
 * is up to the user.
 *===========================================================================
 */
void StateAtBirth(double *istate[POPULATION_NR], int BirthStateNr, double E[])
{
    AGE       = 0.0;
    LENGTH    = LB;
    WEIGHT    = BETA*pow(LB,3);
    
    return;
}
/*
 *===========================================================================
 * Specify the threshold determining the end point of each discrete life
 * stage in individual life history as function of the i-state variables and
 * the individual's state at birth for all populations in every life stage.
 *
 * Notice that the first index of the variable 'istate[][]' refers to the
 * number of the structured population, the second index refers to the
 * number of the individual state variable. The interpretation of the latter
 * is up to the user.
 *===========================================================================
 */

void IntervalLimit(int lifestage[POPULATION_NR], double *istate[POPULATION_NR],
                   double *birthstate[POPULATION_NR], int BirthStateNr, double E[],
                   double limit[POPULATION_NR])
{
    switch (lifestage[0])
    {
        case 0:
            limit[0] = LENGTH - LV;  //Second R available
            break;
        case 1:
            limit[0] = LENGTH - LV-LC;  //META
            break;
        case 2:
            limit[0] = LENGTH - LJ;  //MATU
            break;
        case 3:
            limit[0] = AGE - MAX_AGE;
            break;
    }
    
    return;
}



/*
 *===========================================================================
 * Specify the individual development of individuals as function of i-state
 * and environment for all individuals of all populations in every life stage
 *
 * Notice that the first index of the variables 'istate[][]' and 'growth[][]'
 * refers to the number of the structured population, the second index refers
 * to the number of the individual state variable. The interpretation of the
 * latter is up to the user.
 *===========================================================================
 */

void Development(int lifestage[POPULATION_NR], double *istate[POPULATION_NR],
                 double *birthstate[POPULATION_NR], int BirthStateNr, double E[],
                 double development[POPULATION_NR][I_STATE_DIM])
{
    
    
    double        A1, A2, PSI_A, PHI;
    double        MAX_EXP = 50.0;
    double		pw;
     double        GrowthStop, InstDeath, CritL, Growth;
     double        Kappa=0.7;
    
    PSI_A = min(1, PSI+META);                       //Specialization X1 J&A
    
    
    if (lifestage[0] == 0)
    {
        A1 = (1-PSI)*(AMAX-AMIN)+AMIN;
        A2 = 0;
        GrowthStop = LM*H*A1*X1/(1+A1*H*X1);
        InstDeath = GrowthStop/Kappa;
        CritL = InstDeath-Tqs*(InstDeath-GrowthStop);
        
    }
    else if (lifestage[0] ==1)
    {
        A1=(1-PSI)*(AMAX-AMIN)+AMIN;
        A2=(PSI)*(AMAX-AMIN)+AMIN;
        pw = -SIGMA * (A1  * X1 - A2  * X2);
         pw = max(pw, -MAX_EXP);
         pw = min(pw,  MAX_EXP);
         pw = exp(pw);
         PHI = 1 / (1 + pw);
         GrowthStop = (PHI*LM*H*A1*X1+(1-PHI)*LM*H*A2*X2)/(1+(1-PHI)*A2*H*X2+PHI*A1*H*X1);
        InstDeath = GrowthStop/Kappa;
         CritL = InstDeath-Tqs*(InstDeath-GrowthStop);
    }
    else
    {
        A1 = (1-PSI_A)*(AMAX-AMIN)+AMIN;
        A2 = (PSI_A)*(AMAX-AMIN)+AMIN;
        pw = -SIGMA * (A1  * X1 - A2  * X2);
         pw = max(pw, -MAX_EXP);
         pw = min(pw,  MAX_EXP);
         pw = exp(pw);
         PHI = 1 / (1 + pw);
        GrowthStop = (PHI*LM*H*A1*X1+(1-PHI)*LM*H*A2*X2)/(1+(1-PHI)*A2*H*X2+PHI*A1*H*X1);
        InstDeath = GrowthStop/Kappa;
        CritL = InstDeath-Tqs*(InstDeath-GrowthStop);
    }
    
    
    pw = -SIGMA * (A1  * X1 - A2  * X2);
    pw = max(pw, -MAX_EXP);
    pw = min(pw,  MAX_EXP);
    pw = exp(pw);
    PHI = 1 / (1 + pw);
    
    if (LENGTH>InstDeath) {
        development[0][0]=1000;
    }
    else {
    development[0][0] = 1.0;
    }
    
    switch (lifestage[0])
    {
        case 0:
             if (LENGTH<=GrowthStop){
            Growth=GAMMA*(LM*H*A1*X1/(1+A1*H*X1) - LENGTH);
            }
             else {
                 Growth = 0;
             }
            development[0][1] = Growth;
            development[0][2] = 3*BETA*LENGTH*LENGTH*development[0][1];
            break;
        case 1:
             if (LENGTH<=GrowthStop){
               Growth=GAMMA*((PHI*LM*H*A1*X1+(1-PHI)*LM*H*A2*X2)/(1+(1-PHI)*A2*H*X2+PHI*A1*H*X1)-LENGTH);
             }
              else {
                 Growth = 0;
             }
            development[0][1] = Growth;
            development[0][2] = 3*BETA*LENGTH*LENGTH*development[0][1];
            break;
        case 2:
             if (LENGTH<=GrowthStop){
                Growth=GAMMA*((PHI*LM*H*A1*X1+(1-PHI)*LM*H*A2*X2)/(1+(1-PHI)*A2*H*X2+PHI*A1*H*X1)-LENGTH);
             }
             else {
                Growth = 0;
            }
            development[0][1] = Growth;
            development[0][2] = 3*BETA*LENGTH*LENGTH*development[0][1];
            break;
        case 3:
             if (LENGTH<=GrowthStop){
              Growth=GAMMA*((PHI*LM*H*A1*X1+(1-PHI)*LM*H*A2*X2)/(1+(1-PHI)*A2*H*X2+PHI*A1*H*X1)-LENGTH);
             }
             else {
                Growth = 0;
             }
            development[0][1] = Growth;
            development[0][2] = 3*BETA*LENGTH*LENGTH*development[0][1];
            break;
    }
    
    

    
    
    return;
}


/*
 *===========================================================================
 * Specify the possible discrete changes (jumps) in the individual state
 * variables when ENTERING the stage specified by 'lifestage[]'.
 *
 * Notice that the first index of the variables 'istate[][]' and 'growth[][]'
 * refers to the number of the structured population, the second index refers
 * to the number of the individual state variable. The interpretation of the
 * latter is up to the user.
 *===========================================================================
 */

void DiscreteChanges(int lifestage[POPULATION_NR], double *istate[POPULATION_NR],
                     double *birthstate[POPULATION_NR], int BirthStateNr, double E[])
{
    
    
    
    if (lifestage[0] == 2)
    {
        
        WEIGHT = WEIGHT-WEIGHT*META*QS;// (qs = 1-qs!)
        LENGTH =pow(((WEIGHT)/BETA),0.3333333333333333333);
         SetSurvival(0,((RHO-1)*META+1)*Survival(0)); //
        
        
        
    }
    return;
}


/*
 *===========================================================================
 * Specify the fecundity of individuals as a function of the i-state
 * variables and the individual's state at birth for all populations in every
 * life stage.
 *
 * The number of offspring produced has to be specified for every possible
 * state at birth in the variable 'fecundity[][]'. The first index of this
 * variable refers to the number of the structured population, the second
 * index refers to the number of the birth state.
 *
 * Notice that the first index of the variable 'istate[][]' refers to the
 * number of the structured population, the second index refers to the
 * number of the individual state variable. The interpretation of the latter
 * is up to the user.
 *===========================================================================
 */

void Fecundity(int lifestage[POPULATION_NR], double *istate[POPULATION_NR],
               double *birthstate[POPULATION_NR], int BirthStateNr, double E[],
               double *fecundity[POPULATION_NR])
{
    
    double        A1, A2, PSI_A, PHI;
    double        MAX_EXP = 50.0;
    double		  pw;
     double        GrowthStop, InstDeath, CritL,  fecund;
    double        Kappa=0.7;
    
    PSI_A = min(1, PSI+META);                       //Specialization X1 J&A
    A1 = (1-PSI_A)*(AMAX-AMIN)+AMIN;
    A2 = (PSI_A)*(AMAX-AMIN)+AMIN;

    pw = -SIGMA * (A1  * X1 - A2  * X2);
    pw = max(pw, -MAX_EXP);
    pw = min(pw,  MAX_EXP);
    pw = exp(pw);
    
    PHI = 1 / (1 + pw);
    GrowthStop = (PHI*LM*H*A1*X1+(1-PHI)*LM*H*A2*X2)/(1+(1-PHI)*A2*H*X2+PHI*A1*H*X1);
    InstDeath = GrowthStop/Kappa;
    CritL = InstDeath-Tqs*(InstDeath-GrowthStop);
     
     
     
     if (LENGTH<=GrowthStop){
     fecund = AC * ((PHI*H*A1*X1+(1-PHI)*H*A2*X2)/(1+(1-PHI)*A2*H*X2+PHI*A1*H*X1))*LENGTH*LENGTH;
     }
     else {
     fecund = (AC/(1-Kappa)) * (((PHI*H*A1*X1+(1-PHI)*H*A2*X2)/(1+(1-PHI)*A2*H*X2+PHI*A1*H*X1))-Kappa*LENGTH/LM)*LENGTH*LENGTH;
     }
    
    
    fecundity[0][0] = 0.0;
    if (lifestage[0] == 3) {
        fecundity[0][0] = fecund;
    }
    
    return;
}



/*
 *===========================================================================
 * Specify the mortality of individuals as a function of the i-state
 * variables and the individual's state at birth for all populations in every
 * life stage.
 *
 * Notice that the first index of the variable 'istate[][]' refers to the
 * number of the structured population, the second index refers to the
 * number of the individual state variable. The interpretation of the latter
 * is up to the user.
 *===========================================================================
 */

void Mortality(int lifestage[POPULATION_NR], double *istate[POPULATION_NR],
               double *birthstate[POPULATION_NR], int BirthStateNr, double E[],
               double mortality[POPULATION_NR])
{
    
    double        A1, A2, PSI_A, PHI;
    double        MAX_EXP = 50.0;
    double        pw;
     double        GrowthStop, InstDeath, CritL,  MUTOT;
    double        Kappa=0.7;
    
    PSI_A = min(1, PSI+META);                       //Specialization X1 J&A
    
    
     if (lifestage[0] == 0)
     {
     A1 = (1-PSI)*(AMAX-AMIN)+AMIN;
     A2 = 0;
     GrowthStop = LM*H*A1*X1/(1+A1*H*X1);
     InstDeath = GrowthStop/Kappa;
     CritL = InstDeath-Tqs*(InstDeath-GrowthStop);
     
     }
     else if (lifestage[0] ==1)
     {
     A1=(1-PSI)*(AMAX-AMIN)+AMIN;
     A2=(PSI)*(AMAX-AMIN)+AMIN;
         pw = -SIGMA * (A1  * X1 - A2  * X2);
     pw = max(pw, -MAX_EXP);
     pw = min(pw,  MAX_EXP);
     pw = exp(pw);
     PHI = 1 / (1 + pw);
     GrowthStop = (PHI*LM*H*A1*X1+(1-PHI)*LM*H*A2*X2)/(1+(1-PHI)*A2*H*X2+PHI*A1*H*X1);
     InstDeath = GrowthStop/Kappa;
     CritL = InstDeath-Tqs*(InstDeath-GrowthStop);
     }
     else
     {
     A1 = (1-PSI_A)*(AMAX-AMIN)+AMIN;
     A2 = (PSI_A)*(AMAX-AMIN)+AMIN;
         pw = -SIGMA * (A1  * X1 - A2  * X2);
     pw = max(pw, -MAX_EXP);
     pw = min(pw,  MAX_EXP);
     pw = exp(pw);
     PHI = 1 / (1 + pw);
     GrowthStop = (PHI*LM*H*A1*X1+(1-PHI)*LM*H*A2*X2)/(1+(1-PHI)*A2*H*X2+PHI*A1*H*X1);
     InstDeath = GrowthStop/Kappa;
     CritL = InstDeath-Tqs*(InstDeath-GrowthStop);
     }
     
     if (LENGTH > InstDeath) {
     MUTOT = MU+1000;
     }
     else if (LENGTH > CritL){
          MUTOT = MU+mu_s*(LENGTH-CritL);
     }
     else {
     MUTOT = MU;
     }
    mortality[0] = MUTOT;
    
    
    return;
}

/*
 *===========================================================================
 * For all the integrals (measures) that occur in interactions of the
 * structured populations with their environments and for all the integrals
 * that should be computed for output purposes (e.g. total juvenile or adult
 * biomass), specify appropriate weighing function dependent on the i-state
 * variables, the environment variables and the current life stage of the
 * individuals. These weighing functions should be specified for all
 * structured populations in the problem. The number of weighing functions
 * is the same for all of them.
 *
 * Notice that the first index of the variables 'istate[][]' and 'impact[][]'
 * refers to the number of the structured population, the second index of the
 * variable 'istate[][]' refers to the number of the individual state variable,
 * while the second index of the variable 'impact[][]' refers to the number of
 * the interaction variable. The interpretation of these second indices is up
 * to the user.
 *===========================================================================
 */

void Impact(int lifestage[POPULATION_NR], double *istate[POPULATION_NR],
            double *birthstate[POPULATION_NR], int BirthStateNr, double E[],
            double impact[POPULATION_NR][INTERACT_DIM])
{
    
    double        A1, A2, PSI_A, PHI;
    double        MAX_EXP = 50.0;
    double		pw;
    
    
    PSI_A = min(1, PSI+META);                       //Specialization X1 J&A
    
    
    if (lifestage[0] == 0)
    {
        A1 = (1-PSI)*(AMAX-AMIN)+AMIN;
        A2 = 0;
    }
    else if (lifestage[0] ==1)
    {
        A1=(1-PSI)*(AMAX-AMIN)+AMIN;
        A2=(PSI)*(AMAX-AMIN)+AMIN;
    }
    else
    {
        A1 = (1-PSI_A)*(AMAX-AMIN)+AMIN;
        A2 = (PSI_A)*(AMAX-AMIN)+AMIN;
    }
    
    
    pw = -SIGMA * (A1  * X1 - A2  * X2);
    pw = max(pw, -MAX_EXP);
    pw = min(pw,  MAX_EXP);
    pw = exp(pw);
    
    PHI = 1 / (1 + pw);
    
    
    switch (lifestage[0])
    {
        case 0:
            impact[0][ 0] = (A1*X1/(1+A1*H*X1))*LENGTH*LENGTH;
            impact[0][ 1] = 0;
            impact[0][ 2] = 1;
            impact[0][ 3] = BETA*LENGTH*LENGTH*LENGTH;
            impact[0][ 4] = 0;
            impact[0][ 5] = 0;
            impact[0][ 6] = 0;
            impact[0][ 7] = 0;
            impact[0][ 8] = 0;
            impact[0][ 9] = 0;

            break;
        case 1:
            impact[0][ 0] = (PHI*A1*X1/(1+PHI*A1*H*X1+(1-PHI)*A2*H*X2))*LENGTH*LENGTH;
            impact[0][ 1] = ((1-PHI)*A2*X2/(1+PHI*A1*H*X1+(1-PHI)*A2*H*X2))*LENGTH*LENGTH;
            impact[0][ 2] = 0;
            impact[0][ 3] = 0;
            impact[0][ 4] = 1;
            impact[0][ 5] = BETA*LENGTH*LENGTH*LENGTH;
            impact[0][ 6] = 0;
            impact[0][ 7] = 0;
            impact[0][ 8] = 0;
            impact[0][ 9] = 0;

            break;
        case 2:
            impact[0][ 0] = (PHI*A1*X1/(1+PHI*A1*H*X1+(1-PHI)*A2*H*X2))*LENGTH*LENGTH;
            impact[0][ 1] = ((1-PHI)*A2*X2/(1+PHI*A1*H*X1+(1-PHI)*A2*H*X2))*LENGTH*LENGTH;
            impact[0][ 2] = 0;
            impact[0][ 3] = 0;
            impact[0][ 4] = 0;
            impact[0][ 5] = 0;
            impact[0][ 6] = 1;
            impact[0][ 7] = BETA*LENGTH*LENGTH*LENGTH;
            impact[0][ 8] = 0;
            impact[0][ 9] = 0;

            break;
        case 3:
            impact[0][ 0] = (PHI*A1*X1/(1+PHI*A1*H*X1+(1-PHI)*A2*H*X2))*LENGTH*LENGTH;
            impact[0][ 1] = ((1-PHI)*A2*X2/(1+PHI*A1*H*X1+(1-PHI)*A2*H*X2))*LENGTH*LENGTH;
            impact[0][ 2] = 0;
            impact[0][ 3] = 0;
            impact[0][ 4] = 0;
            impact[0][ 5] = 0;
            impact[0][ 6] = 0;
            impact[0][ 7] = 0;
            impact[0][ 8] = 1;
            impact[0][ 9] = BETA*LENGTH*LENGTH*LENGTH;

            break;
    }
   
    return;
}







/*
 *===========================================================================
 * Specify the type of each of the environment variables by setting
 * the entries in EnvironmentType[ENVIRON_DIM] to PERCAPITARATE, GENERALODE
 * or POPULATIONINTEGRAL based on the classification below:
 *
 * Set an entry to PERCAPITARATE if the dynamics of E[j] follow an ODE and 0
 * is a possible equilibrium state of E[j]. The ODE is then of the form
 * dE[j]/dt = P(E,I)*E[j], with P(E,I) the per capita growth rate of E[j].
 * Specify the equilibrium condition as condition[j] = P(E,I), do not include
 * the multiplication with E[j] to allow for detecting and continuing the
 * transcritical bifurcation between the trivial and non-trivial equilibrium.
 *
 * Set an entry to GENERALODE if the dynamics of E[j] follow an ODE and 0 is
 * NOT an equilibrium state of E. The ODE then has a form dE[j]/dt = G(E,I).
 * Specify the equilibrium condition as condition[j] = G(E,I).
 *
 * Set an entry to POPULATIONINTEGRAL if E[j] is a (weighted) integral of the
 * population distribution, representing for example the total population
 * biomass. E[j] then can be expressed as E[j] = I[p][i]. Specify the
 * equilibrium condition in this case as condition[j] = I[p][i].
 *
 * Notice that the first index of the variable 'I[][]' refers to the
 * number of the structured population, the second index refers to the
 * number of the interaction variable. The interpretation of the latter
 * is up to the user. Also notice that the variable 'condition[j]' should
 * specify the equilibrium condition of environment variable 'E[j]'.
 *===========================================================================
 */

const int EnvironmentType[ENVIRON_DIM] = {GENERALODE};

void EnvEqui(double E[], double I[POPULATION_NR][INTERACT_DIM],
             double condition[ENVIRON_DIM])
{
    condition[0] = DELTA*(XMAX1-X1) - I[0][0];
    condition[1] = DELTA*(XMAX2-X2) - I[0][1];
    
    
    return;
}





